use std::{io::{Cursor, Seek, SeekFrom}, fmt::Display};

use anyhow::Result;

use crate::packets::{Decode, Encode};

use super::varint::VarInt;

#[derive(Debug, PartialEq, Eq)]
pub struct LenghtPrefixedString {
    pub value: String,
}

impl Decode for LenghtPrefixedString {
    fn decode(cursor: &mut Cursor<&[u8]>) -> Result<LenghtPrefixedString> {
        let lenght: i32 = VarInt::decode(cursor)?.into();
        let value = String::from_utf8(cursor.remaining_slice()[..lenght as usize].to_owned())?;
        cursor.seek(SeekFrom::Current(lenght as i64))?;

        Ok(LenghtPrefixedString { value })
    }
}

impl Encode for LenghtPrefixedString {
    fn encode(&self, buffer: &mut Vec<u8>) -> Result<()> {
        let lenght = VarInt {value: self.value.len() as i32};
        lenght.encode(buffer)?;

        buffer.extend_from_slice(self.value.as_bytes());
        Ok(())
    }
}


impl From<LenghtPrefixedString> for String {
    fn from(val: LenghtPrefixedString) ->  Self {
        val.value
    }
}

impl Display for LenghtPrefixedString {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        self.value.fmt(f)
    }
}
