use std::io::Cursor;

use anyhow::Result;

use crate::{datatypes::lenght_prefixed_string::LenghtPrefixedString, packets::{Decode, Encode, Packet}};


pub struct StatusResponsePacket {
    pub json: LenghtPrefixedString,
}

impl Decode for StatusResponsePacket {
    fn decode(cursor: &mut Cursor<&[u8]>) -> Result<Self> {
        let json = LenghtPrefixedString::decode(cursor)?;

        Ok(StatusResponsePacket { json })
    }
}

impl Encode for StatusResponsePacket {
    fn encode(&self, buffer: &mut Vec<u8>) -> Result<()> {
        self.json.encode(buffer)
    }
}

impl Packet for StatusResponsePacket {
    fn get_id(&self) -> i32 {0x00}
}


pub struct StatusPongResponsePacket {
    pub payload: i64,
}

impl Decode for StatusPongResponsePacket {
    fn decode(cursor: &mut Cursor<&[u8]>) -> Result<Self> {
        let payload = i64::decode(cursor)?;

        Ok(StatusPongResponsePacket { payload })
    }
}

impl Encode for StatusPongResponsePacket {
    fn encode(&self, buffer: &mut Vec<u8>) -> anyhow::Result<()> {
        self.payload.encode(buffer)
    }
} 

impl Packet for StatusPongResponsePacket {
    fn get_id(&self) -> i32 {0x01}
}
