use uuid::Uuid;

use crate::{datatypes::lenght_prefixed_string::LenghtPrefixedString, packets::{Decode, Encode, Packet}};


pub struct LoginStartPacket {
    pub username: LenghtPrefixedString,
    pub has_uuid: bool,
    pub uuid: Option<Uuid>,
}

impl Decode for LoginStartPacket {
    fn decode(cursor: &mut std::io::Cursor<&[u8]>) -> anyhow::Result<Self> {
        let username = LenghtPrefixedString::decode(cursor)?;
        let has_uuid = bool::decode(cursor)?;
        let uuid = if has_uuid {
            Some(Uuid::decode(cursor)?)
        } else {
            None
        };

        Ok(LoginStartPacket {
            username,
            has_uuid,
            uuid,
        })
    }
}

impl Encode for LoginStartPacket {
    fn encode(&self, buffer: &mut Vec<u8>) -> anyhow::Result<()> {
        self.username.encode(buffer)?;
        self.has_uuid.encode(buffer)?;
        self.uuid.encode(buffer)?;

        Ok(())
    }
}

impl Packet for LoginStartPacket {
    fn get_id(&self) -> i32 {0x00}
}
